@@ -7,6 +7,7 @@ import numpy as np
 import matplotlib.pyplot as plt
 from .private_func import _preprocessing, _corr2
 import warnings
+from .backtest import get_pnl_plot
 
 warnings.filterwarnings("ignore")
 
@@ -17,9 +18,6 @@ class simpletest():
     """
 
     def __init__(self):
-        """
-        initialization
-        """
         pass
 
     def get_cdf_peak(self, x, y):
@@ -42,7 +40,9 @@ class simpletest():
         return peak
 
     def simple_pnl(self, x, y,
-                   ax=None, plot=True, data_return=False, sign=True, med=True, peak=False):
+                   ax=None, st_plot=True, data_return=True,
+                   sign=True, med=True, peak=True,
+                   bt_plot=False, **kwargs):
         """
         x: pd.Series
         y: pd.Series
@@ -65,12 +65,16 @@ class simpletest():
             return
 
         factor, ret = _preprocessing(x, y)
-        factor_mid = np.median(factor)
 
         corr = _corr2(factor, ret)
 
         output = {}
 
+        if st_plot:
+            if ax is None:
+                fig = plt.figure(figsize=(15, 6))
+                ax = fig.add_subplot(111)
+
         if sign:
             indicator1 = np.sign(factor) * 100 * np.sign(corr)
             label1 = 'Sign-Trade: '
@@ -89,10 +93,7 @@ class simpletest():
             # year PnL
             pnl_y1 = day_pnl1.mean() * 251
 
-            if plot:
-                if ax is None:
-                    plt.figure(figsize=(15, 6))
-                    ax = plt.gca()
+            if st_plot:
                 sum_pnl1.plot(ax=ax, label=label1 +
                                            'PnL={:.2f} Sharpe={:.2f} PoT={:.2f}'
                               .format(pnl_y1, sharpe1, pot1))
@@ -102,7 +103,8 @@ class simpletest():
                 output['delta_sign'] = indicator1
 
         if med:
-            indicator2 = factor.apply(lambda m: 1 if m > factor_mid else -1) * 100 * np.sign(corr)
+            factor_mid = np.median(factor)
+            indicator2 = factor.apply(lambda m: 1 if m >= factor_mid else -1) * 100 * np.sign(corr)
             label2 = 'Med-Trade: '
 
             pnl2 = ret * indicator2
@@ -119,9 +121,7 @@ class simpletest():
             # year PnL
             pnl_y2 = day_pnl2.mean() * 251
 
-            if plot:
-                if ax is None:
-                    ax = plt.gca()
+            if st_plot:
                 sum_pnl2.plot(ax=ax, label=label2 +
                                            'PnL={:.2f} Sharpe={:.2f} PoT={:.2f}'
                               .format(pnl_y2, sharpe2, pot2))
@@ -131,8 +131,8 @@ class simpletest():
                 output['delta_med'] = indicator2
 
         if peak:
-            peak_val = self.get_cdf_peak(x, y)
-            indicator3 = factor.apply(lambda m: 1 if m > peak_val else -1) * 100 * np.sign(corr)
+            peak_val = self.get_cdf_peak(factor, ret)
+            indicator3 = factor.apply(lambda m: 1 if m >= peak_val else -1) * 100 * np.sign(corr)
             label3 = 'Peak-Trade: '
 
             pnl3 = ret * indicator3
@@ -149,9 +149,7 @@ class simpletest():
             # year PnL
             pnl_y3 = day_pnl3.mean() * 251
 
-            if plot:
-                if ax is None:
-                    ax = plt.gca()
+            if st_plot:
                 sum_pnl3.plot(ax=ax, label=label3 +
                                            'PnL={:.2f} Sharpe={:.2f} PoT={:.2f}'
                               .format(pnl_y3, sharpe3, pot3))
@@ -160,7 +158,7 @@ class simpletest():
                 output['pnl_peak'] = sum_pnl3
                 output['delta_peak'] = indicator3
 
-        if plot:
+        if st_plot:
             if ax is None:
                 ax = plt.gca()
             (y.cumsum() * 100.0).dropna().plot(ax=ax,
@@ -171,12 +169,25 @@ class simpletest():
             ax.set_xlabel('Date')
             ax.legend()
 
+        # back_test pnl plot
+        if bt_plot:
+            if sign:
+                get_pnl_plot(indicator1, ret, **kwargs)
+
+            if med:
+                get_pnl_plot(indicator2, ret, **kwargs)
+
+            if peak:
+                get_pnl_plot(indicator3, ret, **kwargs)
+
         if data_return:
             return output
 
         return
 
-    def threshold_pnl(self, x, y, t_list=None, sign_list=None, ax=None, plot=True, data_return=False):
+    def threshold_pnl(self, x, y, t_list=None, sign_list=None,
+                      ax=None, st_plot=False, data_return=True,
+                      bt_plot=True, **kwargs):
         """
         x: pd.Series
         y: pd.Series
@@ -227,7 +238,7 @@ class simpletest():
             else:
                 t_left = t_list[i - 1]
                 t_right = t_list[i]
-            indicator.loc[factor >= t_left & factor < t_right] = sign_list[i] * 100.0
+            indicator.loc[(factor >= t_left) & (factor < t_right)] = sign_list[i] * 100.0
 
         pnl = ret * indicator
         sum_pnl = pnl.cumsum()
@@ -243,7 +254,7 @@ class simpletest():
         # year PnL
         pnl_y = day_pnl.mean() * 250
 
-        if plot:
+        if st_plot:
             if ax is None:
                 plt.figure(figsize=(15, 6))
                 ax = plt.gca()
@@ -257,6 +268,9 @@ class simpletest():
             ax.set_xlabel('Date')
             ax.legend()
 
+        if bt_plot:
+            get_pnl_plot(indicator, ret, **kwargs)
+
         if data_return:
             return {'pnl': sum_pnl, 'delta': indicator}
 
@@ -410,6 +424,42 @@ class simpletest():
             plt.title(data.value_counts().sort_index(
                 ascending=False).reindex(list_value).fillna(0.0).to_string().replace('\n', '; ').replace('    ', ':'))
 
+    def plot_quantile_ret(self, x, y, ax=None, is_plot=True):
+        x = x.copy()
+        pctl = [0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9]
+
+        qtl = x.quantile(pctl).drop_duplicates()
+        pctl = qtl.index
+        x_q = list(sorted(qtl))
+
+        range_list = [-np.inf] + x_q + [np.inf]
+        range_list_name = [F'0.0,{pctl[0]}']
+        for i in range(0, len(pctl) - 1):
+            range_list_name.append(F'{pctl[i]},{pctl[i + 1]}')
+        range_list_name.append(F'{pctl[-1]},1.0')
+
+        if len(set(range_list)) == len(range_list):
+            binned_x = pd.cut(x, range_list, labels=range_list_name)
+            Y_G = y.groupby(by=binned_x.values)
+            y_mean = Y_G.mean().T * 10000
+            y_mean.columns = range_list_name
+        else:
+            X = x.values
+            y_mean = {}
+            for i in range(len(range_list) - 1):
+                y_mean[range_list_name[i]] = y.loc[(range_list[i] < X) & (X <= range_list[i + 1])].mean() * 10000
+            y_mean = pd.DataFrame(y_mean)
+
+        if is_plot:
+            if ax is None:
+                plt.figure(figsize=(10, 10))
+                ax = plt.gca()
+
+            color_list = y_mean.apply(lambda m: 'r' if m >= 0 else 'g')
+            ax.bar(y_mean.index, y_mean.values, color=color_list)
+            ax.set_xlabel('Quantile')
+            ax.set_ylabel('BPS')
+
     def hist(self, x, bins=100, ax=None, describe=False):
         if ax is None:
             plt.figure(figsize=(10, 10))
@@ -423,8 +473,20 @@ class simpletest():
         ax.set_ylabel('Count')
         ax.set_title('Factor Hist')
 
+    def ic(self, x, y):
+        return pd.concat([x, y], axis=1).corr(method='spearman').iloc[0, 1]
+
+    def ir(self, x, y, window=250):
+        def _rolling_ic(m):
+            return pd.concat([m, y.loc[m.index]],
+                             axis=1).corr(method='spearman').iloc[0, 1]
+
+        rank_ic = x.rolling(window).apply(_rolling_ic)
+
+        return rank_ic.mean()/rank_ic.std()
+
     def plot_composite(self, x, y, markout_periods=30, cdf_period2=5, fig_return=False,
-                       sign=True, med=True, peak=True, **kwargs):
+                       **kwargs):
         """
         x, y: prefer pd.Series
         x: factor
@@ -443,39 +505,68 @@ class simpletest():
             print('index of x and y must be pd.core.indexes.datetimes.DatetimeIndex')
             return
 
+        if 'ic' in kwargs:
+            if kwargs.pop('ic'):
+                print('IC: ', self.ic(x, y))
+
+        if 'ir' in kwargs:
+            if kwargs.pop('ir'):
+                print('IR: ', self.ir(x, y))
+
         x, y = _preprocessing(x, y)
 
         if 'figsize' in kwargs:
             figsize = kwargs.pop('figsize')
         else:
-            figsize = (20, 20)
+            figsize = (16, 24)
 
         fig = plt.figure(figsize=figsize)
 
         if 'title' in kwargs:
             title = kwargs.pop('title')
-            plt.title(str(title))
-            _ = plt.xticks([])
-            _ = plt.yticks([])
+            ax = plt.gca()
+            ax.set_title(str(title))
+            _ = ax.set_xticks([])
+            _ = ax.set_yticks([])
 
         Y = pd.concat([y.rolling(i).sum().shift(-i + 1).rename(str(i)) \
                        for i in range(1, markout_periods + 1)], axis=1)
 
-        ax1 = fig.add_subplot(221)
+        ax1 = plt.subplot2grid((3, 2), (0, 0), fig=fig)
         self.plot_cdf(x, y, ax=ax1)
         ax1.set_title('CDF 1Period')
 
-        ax2 = fig.add_subplot(222)
+        ax2 = plt.subplot2grid((3, 2), (0, 1), fig=fig)
         self.plot_cdf(x, y.shift(1).rolling(cdf_period2).sum().shift(-cdf_period2), ax=ax2)
         ax2.set_title('CDF {}Period'.format(cdf_period2))
 
-        ax3 = fig.add_subplot(223)
+        ax3 = plt.subplot2grid((3, 2), (1, 0), fig=fig)
         self.plot_markout_quantile(x, Y, ax=ax3, islegend=True)
         ax3.set_title('Markout')
         ax3.legend()
 
-        ax4 = fig.add_subplot(224)
-        self.simple_pnl(x, y, ax=ax4, plot=True, sign=sign, med=med, peak=peak)
+        ax4 = plt.subplot2grid((3, 2), (1, 1), fig=fig)
+        self.plot_quantile_ret(x, y, ax=ax4)
+        ax4.set_title('Quantile Ret Mean')
+
+        if 'sign' in kwargs:
+            sign = kwargs.pop('sign')
+        else:
+            sign = True
+
+        if 'med' in kwargs:
+            med = kwargs.pop('med')
+        else:
+            med = True
+
+        if 'peak' in kwargs:
+            peak = kwargs.pop('peak')
+        else:
+            peak = True
+
+        ax5 = plt.subplot2grid((3, 2), (2, 0), colspan=2, fig=fig)
+        self.simple_pnl(x, y, ax=ax5, st_plot=True, bt_plot=False,
+                        sign=sign, med=med, peak=peak)
         plt.show()
 
         if fig_return:
